<?php

declare(strict_types=1);

namespace Drupal\Tests\taxonomy_entity_index\Functional;

use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Defines a class for testing reindexing form.
 *
 * @group taxonomy_entity_index
 */
final class ReindexTest extends BrowserTestBase {

  use UserCreationTrait;
  use TaxonomyTestTrait;
  use NodeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'taxonomy',
    'taxonomy_entity_index',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalCreateContentType(['type' => 'article']);
    FieldStorageConfig::create([
      'field_name' => 'tags',
      'type' => 'entity_reference',
      'entity_type' => 'node',
      'settings' => [
        'target_type' => 'taxonomy_term',
      ],
    ])->save();
    FieldConfig::create([
      'field_name' => 'tags',
      'bundle' => 'article',
      'entity_type' => 'node',
    ])->save();
  }

  /**
   * Tests reindex form.
   */
  public function testReindexForm(): void {
    $vocab = $this->createVocabulary(['vid' => 'tags']);
    $tag = $this->createTerm($vocab, ['name' => 'Tag']);
    $node = $this->createNode(['type' => 'article', 'tags' => $tag]);
    $this->assertEquals(0, \Drupal::database()
      ->select('taxonomy_entity_index')
      ->condition('entity_type', 'node')
      ->condition('entity_id', $node->id())
      ->countQuery()
      ->execute()
      ->fetchField());

    \Drupal::configFactory()->getEditable('taxonomy_entity_index.settings')->set('types', ['node' => 'node'])->save();

    $url = Url::fromRoute('taxonomy_entity_index.admin_reindex');
    $this->drupalGet($url);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalLogin($this->createUser([
      'administer site configuration',
    ]));
    $this->drupalGet($url);
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'types[node]' => TRUE,
    ], 'Rebuild index');
    $this->assertEquals(1, \Drupal::database()
      ->select('taxonomy_entity_index')
      ->condition('entity_type', 'node')
      ->condition('entity_id', $node->id())
      ->countQuery()
      ->execute()
      ->fetchField());
  }

}
